// KursVirtTable.cpp : Defines the entry point for the console application.
//

#include <iostream>
#include <tchar.h>

#include "tb.h"
#include "debug.h"

void showtable(tb_table& table,int cols)
{
  for(int j=0;j<cols;j++)
  {
    tb_column col;
    tb_getcolumn(&table,j,&col);
    std::cout<<col.name<<"\t";
  }

  std::cout<<std::endl<<"---"<<std::endl;

  for(int i=0;i<tb_getnrows(&table);i++)
  {
    for(int j=0;j<cols;j++)
    {
      tb_column col;
      tb_getcolumn(&table,j,&col);

      char buf[1024];

      switch(col.type)
      {
      case tb_type_int:
        std::cout<<tb_getint(&table,i,j);
        break;
      case tb_type_char:
        std::cout<<tb_getchar(&table,i,j,buf,sizeof(buf));
        break;
      case tb_type_double:
        std::cout<<tb_getdouble(&table,i,j);
        break;
      }
      std::cout<<"\t";
    }
    std::cout<<std::endl;
  }
  std::cout<<std::endl;
}

#define ISLOLETTER(x) (x>='a'&&x<='z')
#define ISHILETTER(x) (x>='A'&&x<='Z')
#define ISCHAR(x) (ISLOLETTER(x)||ISHILETTER(x))

void loadtable(tb_table &table,const char* text,int cols)
{
  int i=0,j=0;
  tb_append(&table);
  while(1)
  {
    std::string str;
    while(ISCHAR(*text))
    {
      str+=*text;
      text++;
    }

    if(!str.empty())
    {
      if(j>=cols)
      {
        j=0;
        i++;
        if(i>15)
          return;
        tb_append(&table);
      }
      tb_setchar(&table,i,j,str.c_str());
      j++;
    }

    while(!ISCHAR(*text))
    {
      if(*text==0)
        return;
      text++;
    }
  }
}

void testtable()
{
  const char text[]=
    "If you have a dual-boot computer and want to install and run Visual Studio programs with both operating systems, you must install these products into separate directories."
    "Installing a Visual Studio product, such as Visual C++ or Visual Basic, multiple times on the same directory can result in undefined behavior. Installing a product to the same location from each operating system is sometimes done to save disk space, but it will lead to problems and is not supported."
    "For example, if you install onto the same directory, you will not be able to fully uninstall the product from both systems. Uninstalling on one system may result in undefined behavior, including the removal of system files from the other system."
    "Installing Visual Studio from a Shared CD Drive"
    "If you are installing Visual Studio 6.0 from a shared compact disc drive, name the share with less than nine characters and with no embedded spaces in the name."
    "Let Microsoft Internet Explorer Setup Finish Before Proceeding with Visual Studio Installation"
    "During Visual Studio setup, the Installation Wizard will install Internet Explorer 4.01 Service Pack (SP) 1. Near the end of the Internet Explorer setup, the Internet Explorer setup wizard will restart your computer. The Internet Explorer setup wizard will automatically restart and continue copying updated files and configuring your system."
    "On some computers, the Visual Studio Installation Wizard may also start while the Internet Explorer setup wizard is finishing this task. You must let the Internet Explorer setup wizard complete its processing before continuing with Visual Studio setup."
    "Specifically, it is vital that you do not continue with setup to the point of restarting the computer again. Restarting the computer during the final configuration phase of Internet Explorer will corrupt the Visual Studio setup. The Internet Explorer setup wizard shows progress indicators while in its final phase. When these progress indicators disappear, it is safe to proceed with the Visual Studio Installation Wizard. "
    "When Microsoft Internet Explorer Setup Installs Older Versions of MDAC Files"
    "The Microsoft Data Access (MDAC) files installed by Microsoft Internet Explorer 4.0 setup may be older than the MDAC files installed by Visual Studio 6.0 products. If you install Internet Explorer on a computer that already has Visual Studio 6.0, and you get a message asking you to confirm whether you want to install older versions of MDAC files, choose to keep the newer components currently on your computer."
    "When the Installation Wizard Looks for A Web Publishing Wizard Disc"
    "When installing Visual Studio 6.0 or the Visual J++ Technology Preview 2, if you get a dialog box asking you to insert the Web Publishing Wizard disc, close the dialog box and proceed with the installation. "
    "Neither Visual Studio nor Visual J++ ships a Web Publishing Wizard on a separate disc. Users may get this message when they are installing Visual Studio or Visual J++ on computers that have Temp directories set to removable drives, such as an Iomega Jaz drive.";

  tb_table table;
  tb_initialize(&table);
  tb_column columns[]=
  {
    {"col1char",tb_type_char,1024,0},
    {"col2char",tb_type_char,1024,0},
    {"col2char",tb_type_char,1024,0}
  };
  tb_describe(&table,columns,sizeof(columns)/sizeof(tb_column),50);

  loadtable(table,text,3);

  showtable(table,3);

  int j[3]={0,1,2};
  int o[3]={1,1,1};
  tb_sort(&table,j,sizeof(j)/sizeof(int),o);

  showtable(table,3);
}

int _tmain(int argc, _TCHAR* argv[])
{
  tb_table table;
  tb_initialize(&table);
  tb_column columns[]=
  {
    {"col1int",tb_type_int,0,0},
    {"col2double",tb_type_double,0,0},
    {"col2char",tb_type_char,1024,0}
  };
  tb_describe(&table,columns,sizeof(columns)/sizeof(tb_column),50);
  verify(tb_insert(&table,0)==0);
  verify(tb_append(&table)==1);
  verify(tb_getnrows(&table)==2);
  verify(tb_remove(&table,0)==0);
  verify(tb_remove(&table,0)==-1);
  verify(tb_append(&table)==0);
  verify(tb_setint(&table,0,0,1)==1);
  verify(tb_getint(&table,0,0)==1);
  verify(tb_setdouble(&table,0,1,10.2)==10.2);
  verify(tb_getdouble(&table,0,1)==10.2);
  verify(strcmp(tb_setchar(&table,0,2,"abc"),"abc")==0);
  char buf[1024];
  verify(strcmp(tb_getchar(&table,0,2,buf,sizeof(buf)),"abc")==0);
  verify(tb_findcol(&table,columns[0].name)==0);
  verify(tb_findcol(&table,columns[2].name)==2);
  tb_clear(&table);verify(tb_getnrows(&table)==0);

  tb_column columns2[]=
  {
    {"col1int",tb_type_int,sizeof(int)},
    {"col2double",tb_type_double,sizeof(double)},
    {"col2char",tb_type_char,sizeof(char)*TB_MAX_NAME_LENGTH}
  };

  int pos[3];
  tb_match(&table,columns2,sizeof(columns2)/sizeof(tb_column),pos,1);
  tb_column col;
  tb_getcolumn(&table,0,&col);
  tb_discard(&table);

  testtable();

	return 0;
}

/*

*/
